@@ -2,9 +2,11 @@ package Text::Template::Simple;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '0.53';
+$VERSION = '0.60';
 
 use Carp qw( croak );
+use File::Spec;
+
 use Text::Template::Simple::Constants;
 use Text::Template::Simple::Dummy;
 use Text::Template::Simple::Compiler;
@@ -15,8 +17,13 @@ use Text::Template::Simple::Util;
 use Text::Template::Simple::Cache::ID;
 use Text::Template::Simple::Cache;
 use Text::Template::Simple::IO;
-# To be removed ...
-use base qw( Text::Template::Simple::Deprecated );
+
+use base qw(
+   Text::Template::Simple::Base::Compiler
+   Text::Template::Simple::Base::Examine
+   Text::Template::Simple::Base::Include
+   Text::Template::Simple::Base::Parser
+);
 
 my %CONNECTOR = ( # Default classes list
    'Cache'     => 'Text::Template::Simple::Cache',
@@ -26,29 +33,59 @@ my %CONNECTOR = ( # Default classes list
 );
 
 my %DEFAULT = ( # default object attributes
-   delimiters    => [ DELIMS ], # default delimiters
-   cache         =>  0, # use cache or not
-   cache_dir     => '', # will use hdd intead of memory for caching...
-   strict        =>  1, # set to false for toleration to un-declared vars
-   safe          =>  0, # use safe compartment?
-   header        =>  0, # template header. i.e. global codes.
-   add_args      => '', # will unshift template argument list. ARRAYref.
-   warn_ids      =>  0, # warn template ids?
-   iolayer       => '', # I/O layer for filehandles
-   stack         => '',
-   user_thandler => undef, # user token handler callback
-   # TODO: Consider removing these
-   fix_uncuddled =>  0, # do some worst practice?
-   resume        =>  0, # resume on error?
+   delimiters     => [ DELIMS ], # default delimiters
+   cache          =>  0,    # use cache or not
+   cache_dir      => '',    # will use hdd intead of memory for caching...
+   strict         =>  1,    # set to false for toleration to un-declared vars
+   safe           =>  0,    # use safe compartment?
+   header         =>  0,    # template header. i.e. global codes.
+   add_args       => '',    # will unshift template argument list. ARRAYref.
+   warn_ids       =>  0,    # warn template ids?
+   iolayer        => '',    # I/O layer for filehandles
+   stack          => '',    # dump caller stack?
+   user_thandler  => undef, # user token handler callback
+   monolith       =>  0,    # use monolithic template & cache ?
+   include_paths  => [],    # list of template dirs
+   pre_chomp      => CHOMP_NONE,
+   post_chomp     => CHOMP_NONE,
+   # TODO: Consider removing this
+   resume         =>  0,    # resume on error?
 );
 
+my @EXPORT_OK = qw( tts );
+
+sub import {
+   my $class = shift;
+   my $caller = caller;
+   my @args   = @_ or return;
+   my %ok     = map { $_, $_ } @EXPORT_OK;
+
+   foreach my $name ( @args ) {
+      croak "$name isn't a valid import parameter for $class" if not $ok{$name};
+      no strict qw( refs );
+      croak "$name is not defined in $class"      if not defined &{ $name   };
+      my $target = $caller . '::' . $name;
+      croak "$name is already defined in $caller" if     defined &{ $target };
+      *{ $target } = \&{ $name }; # install
+   }
+
+   return;
+}
+
+sub tts {
+   my @args = @_;
+   croak "Nothing to compile!" if ! @args;
+   my @new  = ref $args[0] eq 'HASH' ? @{ shift(@args) } : ();
+   return __PACKAGE__->new( @new )->compile( @args );
+}
+
 sub new {
    my $class = shift;
    my %param = scalar(@_) % 2 ? () : (@_);
    my $self  = [ map { undef } 0 .. MAXOBJFIELD ];
    bless $self, $class;
 
-   LOG( CONSTRUCT => $self->_parser_id . " @ ".(scalar localtime time) )
+   LOG( CONSTRUCT => $self->_class_id . " @ ".(scalar localtime time) )
       if DEBUG();
 
    my $fid;
@@ -65,9 +102,9 @@ sub new {
 
 sub connector {
    my $self = shift;
-   my $id   = shift || croak "connector(): id is missing";
-   my $name = $CONNECTOR{ $id } || croak "connector(): $id is not valid";
-   $name;
+   my $id   = shift             || croak "connector(): id is missing";
+   my $name = $CONNECTOR{ $id } || croak "connector(): invalid id: $id";
+   return $name;
 }
 
 sub cache { shift->[CACHE_OBJECT] }
@@ -75,61 +112,12 @@ sub io    { shift->[IO_OBJECT]    }
 
 sub compile {
    my $self  = shift;
-   my $tmpx  = shift or croak "No template specified";
-   my $param = shift || [];
-   my $opt   = shift || {
-      id       => '', # id is AUTO
-      map_keys => 0,  # use normal behavior
-      chkmt    => 0,  # check mtime of file template?
-   };
-
-   croak "params must be an arrayref!" if not isaref($param);
-   croak "opts must be a hashref!"     if not ishref($opt);
-
-   my $tmp = $self->_examine($tmpx);
-   if ( $opt->{chkmt} ) {
-      if ( $self->[TYPE] eq 'FILE' ) { 
-         $opt->{chkmt} = (stat $tmpx)[STAT_MTIME];
-      }
-      else {
-         LOG( DISABLE_MT => "Disabling chkmt. Template is not a file" )
-            if DEBUG();
-         $opt->{chkmt} = 0;
-      }
-   }
-
-   LOG( COMPILE => $opt->{id} ) if defined $opt->{id} && DEBUG();
-
-   my($CODE, $ok);
-   my $cache_id = '';
-
-   if ( $self->[CACHE] ) {
-      my $method = $opt->{id};
-      my @args   = (! $method || $method eq 'AUTO') ? ( $tmp              )
-                 :                                    ( $method, 'custom' )
-                 ;
-      $cache_id  = $self->connector('Cache::ID')->new->generate( @args );
-
-      if ( $CODE = $self->cache->hit( $cache_id, $opt->{chkmt} ) ) {
-         LOG( CACHE_HIT =>  $cache_id ) if DEBUG();
-         $ok = 1;
-      }
-   }
-
-   $self->cache->id( $cache_id ); # if $cache_id;
-   $self->[FILENAME] = $self->[TYPE] eq 'FILE' ? $tmpx : $self->cache->id;
-
-   if ( not $ok ) {
-      # we have a cache miss; parse and compile
-      LOG( CACHE_MISS => $cache_id ) if DEBUG();
-      my $parsed = $self->_parse( $tmp, $opt->{map_keys}, $cache_id );
-      $CODE      = $self->cache->populate( $cache_id, $parsed, $opt->{chkmt} );
-   }
-
-   my   @args;
-   push @args, @{ $self->[ADD_ARGS] } if $self->[ADD_ARGS];
-   push @args, @{ $param };
-   return $CODE->( @args );
+   my $rv    = $self->_compile( @_ );
+   # we need to reset this to prevent false positives
+   # the trick is: this is set in _compile() and sub includes call _compile()
+   # instead of compile(), so it will only be reset here
+   $self->[COUNTER_INCLUDE] = undef;
+   return $rv;
 }
 
 # -------------------[ P R I V A T E   M E T H O D S ]------------------- #
@@ -143,16 +131,25 @@ sub _init {
    croak fatal('ARGS')   if     $bogus_args;
    croak fatal('DELIMS') if not $ok_delim;
 
-   $self->[TYPE]       = '';
-   $self->[COUNTER]    = 0;
-   $self->[FAKER]      = $self->_output_buffer_var;
-   $self->[FAKER_HASH] = $self->_output_buffer_var('hash');
+   $self->[TYPE]           = '';
+   $self->[COUNTER]        = 0;
+   $self->[FAKER]          = $self->_output_buffer_var;
+   $self->[FAKER_HASH]     = $self->_output_buffer_var('hash');
+   $self->[FAKER_SELF]     = $self->_output_buffer_var('self');
+   $self->[INSIDE_INCLUDE] = -1; # must be -1 not 0
+   $self->[NEEDS_OBJECT]   =  0; # does the template need $self ?
+   $self->[DEEP_RECURSION] =  0; # recursion detector
 
    if ( $self->[USER_THANDLER] ) {
       croak "user_thandler parameter must be a CODE reference"
          if ref($self->[USER_THANDLER]) ne 'CODE';
    }
 
+   if ( $self->[INCLUDE_PATHS] ) {
+      croak "include_paths parameter must be a ARRAY reference"
+         if ref($self->[INCLUDE_PATHS]) ne 'ARRAY';
+   }
+
    $self->[IO_OBJECT] = $self->connector('IO')->new( $self->[IOLAYER] );
 
    if ( $self->[CACHE_DIR] ) {
@@ -166,8 +163,6 @@ sub _init {
    return;
 }
 
-sub _parser_id { __PACKAGE__ . " v$VERSION" }
-
 sub _output_buffer_var {
    my $self = shift;
    my $type = shift || 'scalar';
@@ -175,66 +170,32 @@ sub _output_buffer_var {
             : $type eq 'array' ? []
             :                    \my $fake
             ;
-      $id  .= $$; # . rand() . time;
-      $id   =~ tr/a-zA-Z_0-9//cd;
+   $id  = "$id";
+   $id .= int( rand($$) ); # . rand() . time;
+   $id  =~ tr/a-zA-Z_0-9//cd;
+   $id  =~ s{SCALAR}{SELF}xms if $type eq 'self';
    return '$' . $id;
 }
 
-sub _examine {
-   my $self   = shift;
-   my $tmp    = shift;
-   my $length = 0;
-   my $rv;
-   if ( my $ref = ref($tmp) ) {
-      croak fatal(  NOTGLOB => $ref ) if $ref ne 'GLOB';
-      croak fatal( 'NOTFH'          ) if not  fileno $tmp;
-      # hmmm... require Fcntl; flock $tmp, Fcntl::LOCK_SH() if IS_FLOCK;
-      local $/;
-      $rv = <$tmp>;
-      #flock $tmp, Fcntl::LOCK_UN() if IS_FLOCK;
-      close $tmp; # ??? can this be a user option?
-      $self->[TYPE] = 'GLOB';
+sub _file_exists {
+   # TODO: pass INCLUDE_PATHS to ::IO to move this there
+   my $self = shift;
+   my $file = shift;
+
+   return $file if $self->io->is_file( $file );
+
+   foreach my $path ( @{ $self->[INCLUDE_PATHS] } ) {
+      my $test = File::Spec->catfile( $path, $file );
+      return $test if $self->io->is_file( $test );
    }
-   else {
-      $length = length $tmp;
-      if ( $length  <=  255 and $tmp !~ RE_NONFILE and -e $tmp and not -d _ ) {
-         $self->[TYPE] = 'FILE';
-         $rv = $self->io->slurp($tmp);
-         # we don't really need to set this after getting data
-         $length = length $rv if DEBUG();
-      }
-      else {
-         $self->[TYPE] = 'STRING';
-         $rv = $tmp;
-      }
-   }
-   LOG( EXAMINE => $self->[TYPE]."; LENGTH: $length" ) if DEBUG();
-   return $rv;
+
+   return; # fail!
 }
 
-sub _compiler { shift->[SAFE] ? COMPILER_SAFE : COMPILER }
-
-sub _wrap_compile {
-   my $self   = shift;
-   my $parsed = shift or croak "nothing to compile";
-   LOG( CACHE_ID => $self->cache->id ) if $self->[WARN_IDS] && $self->cache->id;
-   LOG( COMPILER => $self->[SAFE] ? 'Safe' : 'Normal' ) if DEBUG();
-   my($CODE, $error);
-
-   $CODE = $self->_compiler->_compile($parsed);
-
-   if( $error = $@ ) {
-      my $error2;
-      if ( $self->[RESUME] ) {
-         $CODE =  sub {
-                     sprintf ("[%s Fatal Error] %s", $self->_parser_id, $error )
-                  };
-         $error2 = $@;
-      }
-      $error .= $error2 if $error2;
-   }
-
-   return $CODE, $error;
+sub _class_id {
+   my $self = shift;
+   my $class = ref($self) || $self;
+   return sprintf( "%s v%s", $class, $self->VERSION() );
 }
 
 sub _tidy {
@@ -265,211 +226,6 @@ sub _tidy {
    return $buf;
 }
 
-sub _parse {
-   my $self     = shift;
-   my $raw      = shift;
-   my $map_keys = shift; # code sections are hash keys
-   my $cache_id = shift;
-
-   my $resume   = $self->[RESUME] || '';
-   my $ds       = $self->[DELIMITERS][DELIM_START];
-   my $de       = $self->[DELIMITERS][DELIM_END  ];
-   my $faker    = $self->[FAKER];
-   my $buf_hash = $self->[FAKER_HASH];
-   my $toke     = $self->connector('Tokenizer')->new( $ds, $de );
-   my $code     = '';
-   my $inside   = 0;
-
-   my($mko, $mkc, $mki);
-
-   if ( $map_keys ) {
-      $mki = $map_keys eq 'init';
-      $mkc = $map_keys eq 'check';
-      $mko = $mki ? MAP_KEYS_INIT
-           : $mkc ? MAP_KEYS_CHECK
-           :        MAP_KEYS_DEFAULT;
-
-      $mko =~ s/<%BUF%>/$faker/xmsg;
-      $mko =~ s/<%HASH%>/$buf_hash/xmsg;
-      $mko =~ s/<%KEY%>/%s/xmsg;
-   }
-
-   LOG( RAW => $raw ) if ( DEBUG() > 3 );
-
-   $self->_fix_uncuddled(\$raw, $ds, $de) if $self->[FIX_UNCUDDLED];
-
-   my $handler = $self->[USER_THANDLER];
-
-   my $w_raw = sub { ";$faker .= q~$_[0]~;" };
-   my $w_cap = sub { ";$faker .= sub {" . $_[0] . "}->();"; };
-
-   # fetch and walk the tree
-   my($id, $str);
-   PARSER: foreach my $token ( @{ $toke->tokenize( $raw, $map_keys ) } ) {
-      ($id, $str) = @{ $token };
-      LOG( TOKEN => "$id => $str" ) if DEBUG() > 3;
-      next PARSER if $id eq 'DISCARD';
-
-      if ( $id eq 'DELIMSTART' ) { $inside++; next PARSER; }
-      if ( $id eq 'DELIMEND'   ) { $inside--; next PARSER; }
-
-      if ( $id eq 'RAW' || $id eq 'NOTADELIM' ) {
-         $code .= $w_raw->($str);
-      }
-      elsif ( $id eq 'CODE' ) {
-         $code .= $resume ? $self->_resume($str, 0, 1) : $str;
-      }
-      elsif ( $id eq 'CAPTURE' ) {
-         $code .= $faker;
-         $code .= $resume ? $self->_resume($str, RESUME_NOSTART)
-                :           " .= sub { $str }->();";
-      }
-      elsif ( $id eq 'DYNAMIC' || $id eq 'STATIC' ) {
-         $code .= $w_cap->( $self->_include($id, $str) );
-      }
-      elsif ( $id eq 'MAPKEY' ) {
-         $code .= sprintf $mko, $mkc ? ( ($str) x 5 ) : $str;
-      }
-      else {
-         if ( $handler ) {
-            LOG( USER_THANDLER => "$id") if DEBUG;
-            $code .= $handler->(
-                        $self, $id ,$str, { capture => $w_cap, raw => $w_raw }
-                     );
-         }
-         else {
-            warn "Adding unknown token as RAW: $id($str)";
-            $code .= $w_raw->($str);
-         }
-      }
-   }
-
-   $self->[FILENAME] ||= '<ANON>';
-
-   if ( $inside ) {
-      my $type = $inside > 0 ? 'opening' : 'closing';
-      my $tmpl = "%d unbalanced %s delimiter(s) in template %s";
-      croak sprintf( $tmpl, abs($inside), $type, $self->[FILENAME] );
-   }
-
-   my $wrapper = '';
-   # build the anonymous sub
-   $wrapper .= "package " . DUMMY_CLASS . ";";
-   $wrapper .= 'use strict;'                   if $self->[STRICT];
-   $wrapper .= 'sub { ';
-   $wrapper .= $self->[HEADER].';'             if $self->[HEADER];
-   $wrapper .= "my $faker = '';";
-   $wrapper .= $self->_add_stack( $cache_id )  if $self->[STACK];
-   $wrapper .= "my $buf_hash = {\@_};"         if $map_keys;
-   $wrapper .= "\n#line 1 " .  $self->[FILENAME] . "\n";
-   $wrapper .= $code . ";return $faker;";
-   $wrapper .= '}';
-
-   LOG( COMPILED => sprintf FRAGMENT_TMP, $self->_tidy($wrapper) )
-      if DEBUG() > 1;
-
-   return $wrapper;
-}
-
-sub _add_stack {
-   my $self    = shift;
-   my $cs_name = shift || '<ANON TEMPLATE>';
-   my $stack   = $self->[STACK] || '';
-
-   return if lc($stack) eq 'off';
-
-   my $check   = ($stack eq '1' || $stack eq 'yes' || $stack eq 'on')
-               ? 'string'
-               : $stack
-               ;
-
-   my($type, $channel) = split /:/, $check;
-   $channel = ! $channel             ? 'warn'
-            :   $channel eq 'buffer' ? $self->[FAKER] . ' .= '
-            :                          'warn'
-            ;
-
-   foreach my $e ( $cs_name, $type, $channel ) {
-      $e =~ s{'}{\\'}xmsg;
-   }
-
-   return "$channel stack( { type => '$type', name => '$cs_name' } );";
-}
-
-sub _include {
-   my $self      = shift;
-   my $type      = shift || '';
-      $type      = lc $type;
-   my $is_static = $type eq 'static';
-   my $is_normal = $type eq 'normal' || $type eq 'dynamic';
-   my $known     = $is_static || $is_normal;
-
-   croak "Unknown include type: $type" if not $known;
-
-   my $file = shift;
-   my $err  = '['.($is_static ? ' static' : '').' include error ]';
-   $file =~ s{\A \s+   }{}xms;
-   $file =~ s{   \s+ \z}{}xms;
-   -e $file  or return "q~$err '$file' does not exist~";
-   -d $file and return "q~$err '$file' is a directory~";
-
-   my $text;
-   LOG( INCLUDE => "$type => '$file'" ) if DEBUG();
-   eval { $text = $self->io->slurp($file) };
-   return "q~$err $@~" if $@;
-
-   if ( $is_normal ) {
-      # creates endless recursive loop if template includes itself
-      # cloning $self can help to overcome this issue
-      return "q~$err dynamic include is disabled. file: '$file'~";
-      $text = $self->_parse($text);
-      return $text;
-   }
-
-   if ( $is_static ) {
-      $text =~ s{\~}{\\~}xmsog;
-      return 'q~'.$text.'~;';
-   }
-
-   return "$err This can not happen!";
-}
-
-sub _resume {
-   my $self    = shift;
-   my $token   = shift           || return;
-   my $nostart = shift           || 0;
-   my $is_code = shift           || 0;
-   my $resume  = $self->[RESUME] || '';
-   my $start   = $nostart ? '' : $self->[FAKER];
-   my $void    = $nostart ? 0  : 1; # not a self-printing block
-
-   if ( $token && $resume && $token !~ RESUME_MY ) {
-      if (
-            $token !~ RESUME_CURLIES &&
-            $token !~ RESUME_ELSIF   &&
-            $token !~ RESUME_ELSE    &&
-            $token !~ RESUME_LOOP
-      ) {
-         LOG( RESUME_OK => $token ) if DEBUG() > 2;
-         my $rvar        = $self->_output_buffer_var('array');
-         my $resume_code = RESUME_TEMPLATE;
-         foreach my $replace (
-            [ RVAR  => $rvar             ],
-            [ TOKEN => $token            ],
-            [ PID   => $self->_parser_id ],
-            [ VOID  => $void             ],
-         ) {
-            $resume_code =~ s{ <% $replace->[0] %> }{$replace->[1]}xmsg;
-         }
-         return $start . $resume_code;
-      }
-   }
-
-   LOG( RESUME_NOT => $token ) if DEBUG() > 2;
-
-   return $is_code ? $token : "$start .= $token;"
-}
-
 sub DESTROY {
    my $self = shift || return;
    LOG( DESTROY => ref $self ) if DEBUG;
@@ -479,49 +235,6 @@ sub DESTROY {
    return;
 }
 
-# Experimental and probably nasty stuff. Re-Write or remove!
-
-sub _fix_uncuddled {
-   my $self = shift;
-   my $tmp  = shift;
-   my $ds   = shift;
-   my $de   = shift;
-   LOG( FIXING => "Worst practice: Cuddling uncuddled else/elsif" );
-   # common part
-   my $start = qr{
-      $ds         # delimiter start
-      (?:\s+|)    # ws or not
-      \}          # block ending
-      (?:\s+|)    # ws or not
-      $de         # delimiter end
-      (?:\s+|)    # ws or not
-      $ds         # delimiter start
-      (?:\s+|)    # ws or not
-   }xms;
-   # fix else
-   $$tmp =~ s{
-      $start
-      else        # keyword
-      (?:\s+|)    # ws or not
-      \{          # block opening
-      (?:\s+|)    # ws or not
-      $de         # delimiter-end
-   }{$ds\} else \{$de}xmsgo;
-   # fix elsif
-   $$tmp =~ s{
-      $start
-      elsif       # keyword
-      (?:\s+|)    # ws or not
-      \( (.+?) \) # elsif bool
-      (?:\s+|)    # ws or not
-      \{          # block opening
-      (?:\s+|)    # ws or not
-      $de
-   }{$ds\} elsif ($1) \{$de}xmsgo;
-   LOG( CUDDLE => "#FIXED\n$$tmp\n#/FIXED" ) if DEBUG() > 2;
-   return;
-}
-
 1;
 
 __END__
@@ -554,9 +267,36 @@ manager.
 =head1 SYNTAX
 
 Template syntax is very simple. There are few kinds of delimiters:
-code blocks (C<< <% %> >>), self-printing blocks (C<< <%= %> >>),
-escaped delimiters (C<< <%! %> >>)
-and static include directive (C<< <%+ %> >>):
+
+=over 4
+
+=item *
+
+Code Blocks: C<< <% %> >>
+
+=item *
+
+Self-printing Blocks: C<< <%= %> >>
+
+=item *
+
+Escaped Delimiters: C<< <%! %> >>
+
+=item *
+
+Static Include Directives: C<< <%+ %> >>
+
+=item *
+
+Dynamic include directives C<< <%* %> >>
+
+=item *
+
+Comment Directives: C<< <%# %> >>
+
+=back
+
+A simple example:
 
    <%
       my @foo = qw(bar baz);
@@ -565,7 +305,7 @@ and static include directive (C<< <%+ %> >>):
    Element is <%= $x %>
    <% } %>
 
-do not directly use print() statements, since they'll break the code.
+Do not directly use print() statements, since they'll break the code.
 Use C<< <%= %> >> blocks. Delimiters can be altered:
 
    $template = Text::Template::Simple->new(
@@ -581,13 +321,129 @@ then you can use them inside templates:
    Element is <?perl= $x ?>
    <?perl } ?>
 
+If you need to remove a code temporarily without deleting, or need to add
+comments:
+
+    <%#
+    This
+    whole
+    block
+    will
+    be
+    ignored
+    %>
+
+If you put a space before the pound sign, the block will be a code block:
+
+   <%
+      # this is normal code not a comment directive
+      my $foo = 42;
+   %>
+
 If you want to include a text or html file, you can use the
 static include directive:
 
    <%+ my_other.html %>
    <%+ my_other.txt  %>
 
-Included files won't be parsed and included statically.
+Included files won't be parsed and included statically. To enable
+parsing for the included files, use the dynamic includes:
+
+   <%* my_other.html %>
+   <%* my_other.txt  %>
+
+Interpolation is also supported with both kind of includes, so the following
+is valid code:
+
+   <%+ "/path/to/" . $txt    %>
+   <%* "/path/to/" . $myfile %>
+
+=head2 Chomping
+
+Chomping is the removal of whitespace before and after your directives. This
+can be useful if you're generating plain text (instead of HTML which'll ignore
+spaces most of the time). You can either remove all space or replace multiple
+whitespace with a single space (collapse). Chomping can be enabled per directive
+or globally via options to the constructor. See L</pre_chomp> and L</post_chomp>
+options to L</new> to globally enable chomping.
+
+Chomping is enabled with second level commands for all directives. Here is
+a list of commands:
+
+   -   Chomp
+   ~   Collapse
+   ^   No chomp (override global)
+
+All directives can be chomped. Here are some examples:
+
+Chomp:
+
+   raw content
+   <%- my $foo = 42; -%>
+   raw content
+   <%=- $foo -%>
+   raw content
+   <%*- /mt/dynamic.tts  -%>
+   raw content
+
+Collapse:
+
+   raw content
+   <%~ my $foo = 42; ~%>
+   raw content
+   <%=~ $foo ~%>
+   raw content
+   <%*~ /mt/dynamic.tts  ~%>
+   raw content
+
+No chomp:
+
+   raw content
+   <%^ my $foo = 42; ^%>
+   raw content
+   <%=^ $foo ^%>
+   raw content
+   <%*^ /mt/dynamic.tts  ^%>
+   raw content
+
+It is also possible to mix the chomping types:
+
+   raw content
+   <%- my $foo = 42; ^%>
+   raw content
+   <%=^ $foo ~%>
+   raw content
+   <%*^ /mt/dynamic.tts  -%>
+   raw content
+
+For example this template:
+
+   Foo
+   <%- $prehistoric = $] < 5.008 -%>
+   Bar
+
+Will become:
+
+   FooBar
+
+And this one:
+
+   Foo
+   <%~ $prehistoric = $] < 5.008 -%>
+   Bar
+
+Will become:
+
+   Foo Bar
+
+Chomping is inspired by Template Toolkit (mostly the same functionality,
+although TT seems to miss collapse/no-chomp per directive option).
+
+=head2 ACCESSING TEMPLATE NAMES
+
+You can use C<$0> to get the template path/name inside the template:
+
+   I am <%= $0 %>
 
 =head2 Escaping Delimiters
 
@@ -624,12 +480,6 @@ You can fetch parameters (passed to compile) in the usual perl way:
    %>
    Baz is <%= $bar{baz} %>
 
-=head2 Special Variables
-
-There is a special variable inside all templates. You must not
-define a variable with the same name inside templates or alter
-it's name before L</compile>.
-
 =head1 METHODS
 
 =head2 new
@@ -775,13 +625,6 @@ If enabled, the module will warn you about compile steps using
 template ids. You must both enable this and the cache. If
 cache is disabled, no warnings will be generated.
 
-=head3 fix_uncuddled
-
-If you are using uncuddled elses/elsifs (which became popular after
-Damian Conway' s PBP Book) in your templates, this will break the parser.
-If you supply this parameter with a true value, the parser will
-reformat the data with cuddled versions before parsing it.
-
 =head3 iolayer
 
 This option does not have any effect under perls older than C<5.8.0>.
@@ -809,6 +652,45 @@ This option is also available to all templates as a function named
 C<stack> for individual stack dumping. See L<Text::Template::Simple::Dummy>
 for more information.
 
+=head3 monolith
+
+Controls the behavior when using includes. If this is enabled, the template
+and all it's includes will be compiled into a single document. If C<monolith>
+is disabled, then the includes will be compiled individually into separate
+documents.
+
+If you need to pass the main template variables (C<my> vars) into dynamic
+includes, then you need to enable this option. However, if you are using the
+cache, then the included templates will not be updated automatically.
+
+C<monolith> is disabled by default.
+
+=head3 include_paths
+
+An ARRAY reference. If you want to use relative file paths when
+compiling/including template files, add the paths of the templates with
+this parameter.
+
+=head3 pre_chomp
+
+   use Text::Template::Simple::Constants qw( :chomp );
+   $pre = CHOMP_NONE; # no chomp
+   $pre = CHOMP_ALL;  # remove all whitespace
+   $pre = COLLAPSE_ALL; # replace all ws with a single space
+   $template = Text::Template::Simple->new(
+      pre_chomp => $pre,
+   );
+
+=head3 post_chomp
+
+   use Text::Template::Simple::Constants qw( :chomp );
+   $post = CHOMP_NONE; # no chomp
+   $post = CHOMP_ALL;  # remove all whitespace
+   $post = COLLAPSE_ALL; # replace all ws with a single space
+   $template = Text::Template::Simple->new(
+      post_chomp => $post,
+   );
+
 =head2 compile DATA [, FILL_IN_PARAM, OPTIONS]
 
 Compiles the template you have passed and manages template cache,
@@ -819,8 +701,9 @@ a reference to a filehandle (C<GLOB>), a string or a file path
 
 =head3 First parameter (DATA)
 
-The first parameter can take three different values; a filehandle,
-a string or a file path. Distinguishing filehandles are easy, since
+The first parameter can take four different values; a filehandle,
+a string, a file path or explicit type definition via an ARRAY reference.
+Distinguishing filehandles are easy, since
 they'll be passed as a reference (but see the bareword issue below).
 So, the only problem is distinguishing strings and file paths. 
 C<compile> first checks if the string length is equal or less than
@@ -832,7 +715,7 @@ text.
 
 You can pass a file path as the first parameter:
 
-   $text = $template->compile('/my/templates/test.tmpl');
+   $text = $template->compile('/my/templates/test.tts');
 
 =head4 Strings
 
@@ -850,7 +733,7 @@ C<GLOB>s must be passed as a reference. If you are using bareword
 filehandles, be sure to pass it's reference or it'll be treated as a 
 file path and your code will probably C<die>:
 
-   open MYHANDLE, '/path/to/foo.tmpl' or die "Error: $!";
+   open MYHANDLE, '/path/to/foo.tts' or die "Error: $!";
    $text = $template->compile(\*MYHANDLE); # RIGHT.
    $text = $template->compile( *MYHANDLE); # WRONG. Recognized as a file path
    $text = $template->compile(  MYHANDLE); # WRONG. Ditto. Will die under strict
@@ -859,16 +742,27 @@ or use the standard C<IO::File> module:
 
    use IO::File;
    my $fh = IO::File->new;
-   $fh->open('/path/to/foo.tmpl', 'r') or die "Error: $!";
+   $fh->open('/path/to/foo.tts', 'r') or die "Error: $!";
    $text = $template->compile($fh);
 
 or you can use lexicals inside C<open> if you don't care about 
 compatibility with older perl:
 
-   open my $fh, '/path/to/foo.tmpl' or die "Error: $!";
+   open my $fh, '/path/to/foo.tts' or die "Error: $!";
    $text = $template->compile($fh);
 
-Filehandles will be automatically closed.
+Filehandles will B<not> be closed.
+
+=head4 Explicit Types
+
+Pass an arrayref containing the type and the parameter to disable guessing
+and forcing the type:
+
+   $text = $template->compile( [ FILE   => '/path/to/my.tts'] );
+   $text = $template->compile( [ GLOB   => \*MYHANDLE] );
+   $text = $template->compile( [ STRING => 'I am running under <%= $] %>'] );
+
+Type can be one of these: C<FILE>, C<GLOB>, C<STRING>.
 
 =head3 FILL_IN_PARAM
 
@@ -989,7 +883,7 @@ compiled into an anonymous sub inside the in-memory cache hash
 and this compiled version will be used instead of continiously
 parsing/compiling the same template.
 
-If disk cache is used, a template file with the "C<.tmpl.cache>"
+If disk cache is used, a template file with the "C<.tts.cache>"
 extension will be generated on the disk.
 
 Using cache is recommended under persistent environments like 
@@ -1017,13 +911,32 @@ Cache keys are generated with one of these modules:
 SHA algorithm seems to be more reliable for key generation, but
 md5 is widely available and C<Digest::MD5> is in CORE.
 
+=head1 FUNCTIONS
+
+=head2 tts [ NEW_ARGS, ] COMPILE_ARGS
+
+This function is a wrapper around the L<Text::Template::Simple> object. It
+creates it's own temporary object behind the scenes and can be used for
+quick Perl one-liners for example. Using this function other than testing is
+not recommended.
+
+C<NEW_ARGS> is optional and must be a hashref containing the parameters to
+L</new>. C<COMPILE_ARGS> is a list and everything it contains will be passed
+to the L</compile> method.
+
+It is possible to import this function to your namespace:
+
+   use Text::Template::Simple qw( tts );
+   print tts("<%= scalar localtime time %>");
+   print tts( { strict => 1 }, "<%= scalar localtime time %>");
+
 =head1 EXAMPLES
 
 TODO
 
 =head1 ERROR HANDLING
 
-You may need to C<eval> your code blocks to trap exceptions. Some 
+You may need to C<eval> your code blocks to trap exceptions. Some recoverable
 failures are silently ignored, but you can display them as warnings 
 if you enable debugging.
 
@@ -1033,7 +946,7 @@ How to add your own tokens into Text::Template::Simple?
 
    use strict;
    use Text::Template::Simple;
-   use constant MYTEMP => q{ Testing: <%$ PROCESS some.tmpl %> };
+   use constant MYTEMP => q{ Testing: <%$ PROCESS some.tts %> };
    # first, register our handler for unknown tokens
    my $t = Text::Template::Simple->new( user_thandler => \&thandler );
    print $t->compile( MYTEMP );
@@ -1090,12 +1003,8 @@ Some methods/functionality of the module needs these optional modules:
 
 =head1 SEE ALSO
 
-For similar functionality, see L<Apache::SimpleTemplate>.
-Also see L<Text::Template> for a similar 
-functionality. L<HTML::Template::Compiled> has a similar approach
-for compiled templates. There is another similar module
-named L<Text::ScriptTemplate>. Also see L<Safe> and
-L<Opcode>.
+L<Apache::SimpleTemplate>, L<Text::Template>, L<Text::ScriptTemplate>,
+L<Safe>, L<Opcode>.
 
 =head1 AUTHOR
 
