@@ -5,7 +5,7 @@ use Text::Template::Simple::Constants;
 use Text::Template::Simple::Util qw( DEBUG LOG ishref binary_mode );
 use Carp qw( croak );
 
-$VERSION = '0.53';
+$VERSION = '0.60';
 
 sub new {
    my $class = shift;
@@ -19,10 +19,12 @@ sub validate {
    my $self = shift;
    my $type = shift || croak "No type specified";
    my $path = shift || croak "No path specified";
+
    if ( $type eq 'dir' ) {
       require File::Spec;
       $path = File::Spec->canonpath( $path );
       my $wdir;
+
       if ( IS_WINDOWS ) {
          $wdir = Win32::GetFullPathName( $path );
          if( Win32::GetLastError() ) {
@@ -34,14 +36,14 @@ sub validate {
             $wdir  = '' if not $ok;
          }
       }
+
       $path = $wdir if $wdir;
       my $ok = -e $path && -d _;
       return if not $ok;
       return $path;
    }
-   else {
-      croak "validate(file) is not yet implemented";
-   }
+
+   croak "validate(file) is not yet implemented";
 }
 
 sub layer {
@@ -58,16 +60,40 @@ sub slurp {
    require Fcntl;
    my $self = shift;
    my $file = shift;
-   my $fh   = IO::File->new;
-   $fh->open($file, 'r') or croak "Error opening $file for reading: $!";
-   flock $fh, Fcntl::LOCK_SH() if IS_FLOCK;
-   $self->layer( $fh );
-   my $tmp  = do { local $/; <$fh> };
+   my($fh, $seek);
+   LOG(IO_SLURP => $file) if DEBUG;
+
+   if ( fileno $file ) {
+      $fh   = $file;
+      $seek = 1;
+   }
+   else {
+      $fh = IO::File->new;
+      $fh->open($file, 'r') or croak "Error opening '$file' for reading: $!";
+   }
+
+   flock $fh,    Fcntl::LOCK_SH()  if IS_FLOCK;
+   seek  $fh, 0, Fcntl::SEEK_SET() if IS_FLOCK && $seek;
+   $self->layer( $fh ) if ! $seek; # apply the layer only if we opened this
+   my $tmp = do { local $/; <$fh> };
    flock $fh, Fcntl::LOCK_UN() if IS_FLOCK;
-   $fh->close;
+   close $fh if ! $seek; # close only if we opened this
    return $tmp;
 }
 
+sub is_file {
+   # safer than a simple "-e"
+   my $self = shift;
+   my $file = shift || return;
+   return     ref $file               ? 0
+         :        $file =~ RE_NONFILE ? 0
+         : length $file >= 255        ? 0
+         : ! -e   $file               ? 0
+         :   -d _                     ? 0
+         :                              1
+         ;
+}
+
 sub DESTROY {
    my $self = shift;
    LOG( DESTROY => ref $self ) if DEBUG;
@@ -93,13 +119,13 @@ TODO
 
 =head1 METHODS
 
-=head2 new PARENT_OBJECT
+=head2 new IO_LAYER
 
-Constructor. Accepts a C<Text::Template::Simple> object as the parameter.
+Constructor. Accepts an I/O layer name as the parameter.
 
 =head2 layer FH
 
-Sets the I/O layer of the supplied filehandle is there is a layer and perl
+Sets the I/O layer of the supplied filehandle if there is a layer and perl
 version is greater or equal to C<5.8>.
 
 =head2 slurp FILE_PATH
@@ -111,6 +137,10 @@ Returns the contents of the supplied file as a string.
 C<TYPE> can either be C<dir> or C<file>. Returns the corrected path if
 it is valid, C<undef> otherwise.
 
+=head2 is_file THING
+
+Test if C<THING> is a file.
+
 =head1 AUTHOR
 
 Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
